from trashtalk.tools import human_readable_from_bytes, print_files


def test_human_readable_from_bytes():
    b = human_readable_from_bytes(500)
    k = human_readable_from_bytes(1024)
    m = human_readable_from_bytes(1024**2)
    g = human_readable_from_bytes(1024**3)
    t = human_readable_from_bytes(1024**4)
    p = human_readable_from_bytes(1024**5)
    e = human_readable_from_bytes(1024**6)
    z = human_readable_from_bytes(1024**7)
    y = human_readable_from_bytes(1024**8)

    assert b == '500'
    assert k == "1K"
    assert m == "1M"
    assert g == "1G"
    assert t == "1T"
    assert p == "1P"
    assert e == "1E"
    assert z == "1Z"
    assert y == "1Y"

    assert "test error" == human_readable_from_bytes("test error")

def test_print_files(capsys):
    l = [["un", 2, 3, 4], ["deux", 2, 3], ["trois", 2, 3, 4]]
    print_files(l, 4)
    out, err = capsys.readouterr()
    assert bool(err) == False
    s = out.split('\n')
    assert s[0] == "un    2  3 4"
    assert s[1] == "deux  2  3  "
    assert s[2] == "trois 2  3 4"

    print_files([], 4)
    out, err = capsys.readouterr()
    assert bool(err) == False
    assert bool(out) == False

    l = [[None, "error"]]
    print_files(l, 4)
    out, err = capsys.readouterr()
    assert bool(out) == False
    assert err == "error\n"



#! /usr/bin/env python3
"""
Option parser and the main function
"""
from __future__ import print_function, absolute_import
import argparse
from trashtalk.generate_trashs import generate_trashs
import sys
from trashtalk.tools import print_files

__all__ = ["trashtalk"]


def parse_option(args=None):
    parser = argparse.ArgumentParser(
        description="Taking out your trash easily")
    # CLASSIC
    from trashtalk.__init__ import __version__
    parser.add_argument('-v', '--version', action='version',
                        version='%(prog)s: ' + __version__)
    parser.add_argument('--verbose', action='store_true')
    # TRASH SELECTION
    selection = parser.add_argument_group('trash selections')
    option = parser.add_argument_group('trash options')
    selection.add_argument('trash', nargs='*', default=[],
                           help=("name where you want use trash, "
                                 "this is a list, you can write home or media,"
                                 " by default your home trash was selected"))
    selection.add_argument('-a', action='store_true', default=False,
                           help="select all trash (home + all your media)")
    selection.add_argument('-u', action='store', nargs='*',
                           help="select user")
    selection.add_argument('-am', action='store_true', default=False,
                           help="select all media, this can depend of user")
    # TRASH OPTION
    option.add_argument('-p', action='store_true', default=False,
                        help="print trash path")
    option.add_argument('-f', '--files', action='store', nargs='*',
                        help="select file in trash")
    option.add_argument('-l', action='store_true', default=False,
                        help="list file in trash")
    option.add_argument('-s', action='store_true',
                        help=("print size"))
    option.add_argument('-i', action='store_true',
                        help=("print info"))
    option.add_argument('-cl', '--clean', action='store_true',
                        help="clean file, or without file all")
    option.add_argument('-rm', action='store', nargs='*',
                        help="move file to selected trash")
    option.add_argument('-re', action='store_true', default=False,
                        help="restore file from selected trash")
    if args:
        return parser.parse_known_args(args)
    return parser.parse_args()


def trashtalk():
    options = parse_option()
    if (options.am or options.trash) and "home" not in options.trash:
        home = False
    else:
        home = True
        if options.trash:
            options.trash.remove('home')
    if options.a:
        options.am = True
    trashs , error = generate_trashs(options.u, options.trash, home, options.am)
    for error in error:
        print(error, file=sys.stderr)
    for trash in trashs:
        error = []
        if not trash:
            continue
        if options.p or (
                not options.l and not
                options.s and not options.clean
                and not options.rm):
            if options.p:
                print('\033[34;1m%s: ' % trash.name, end='')
            print("%s\033[0m" % str(trash))
        if options.l or options.s:
            print_files(
                list(trash.list_files(options.files, options.s, options.i)),
                options.s + options.i * 2 + 1
            )
        if options.clean:
            error = trash.clean(options.files)
        if options.rm:
            error = trash.remove(options.rm)
        if options.re:
            error = trash.restore(options.f)
        if error:
            for e in error:
                print(e, file=sys.stderr)



from __future__ import print_function
from pathlib import Path
from datetime import datetime
from shutil import move
from trashtalk.exception import WrongFormat

__all__ = ["Trash"]


class Trash():
    """
    This object is an implementation of posix trash

    :Example:

    >>> from trasktalk import Trash
    >>> trash = Trash("/path/to/trash", "mytrash")
    >>> for element in trash: # list all elements in this trash
    >>>     print(element)
        element1
        element2
    """

    def __init__(self, path=None, name=""):
        """
        You can init Trash without value or
        path = "The trash path"
        name = "trash name"
        """
        if path:
            self.select_path(path, name)
        else:
            self.name = name

    def select_path(self, path, name):
        self.path = path
        self.name = name
        self.info = str((Path(path) / 'info').absolute())
        self.files = str((Path(path) / 'files').absolute())

    def list_files(self, files=None, size=False, info=False):
        """
        method to list files in trash

        can print size in byte with size
        if file doesn't exist return [None, 'error message']

        :param files: list specific file, if param == None list all elements
        :param size: at true return size in byte with file name
        :return: a generator who yield list [filename], or [filename, size]
        """
        total = 0
        if not files:
            l = Path(self.files).iterdir()
        else:
            l = map(lambda x: (Path(self.files) / x), files)
        for i in l:
            try:
                if not i.exists():
                    raise IOError("File: %s doesn't exist" % i.name)
                list_info_from_file = [i.name]
                if size:
                    list_info_from_file.append(i.lstat().st_size)
                    total += i.lstat().st_size
                if info:
                    i = (Path(self.info) / (i.name + ".trashinfo"))
                    if i.exists():
                        line = list(i.open())
                        for i in range(1, 3):
                            try:
                                list_info_from_file.append(
                                    line[i].split('=')[1].strip())
                            except:
                                list_info_from_file.append("unknow")
                yield list_info_from_file
            except Exception as e:
                yield [None, str(e)]
        if size:
            yield ["Total: ", total]

    def clean(self, list_files=None, path=None):
        """
        method to clean files from trash

        :return: list of error message
        """
        error = []
        info = False
        if not path:
            info = True
            path = self.files
        if not list_files:
            l = Path(path).iterdir()
        else:
            l = map(lambda x: (Path(path) / x), list_files)
        for i in l:
            try:
                if i.is_dir():
                    self.clean(path=(str(i)))
                    i.rmdir()
                else:
                    i.unlink()
                if info:
                    info = (Path(self.info) / (i.name + ".trashinfo"))
                    if info.exists():
                        info.unlink()
            except Exception as e:
                error.append(str(e))
        return error

    def remove(self, list_files=[]):
        """
        move file in list_files to trash and built trashinfo
        """
        if type(list_files) is not list and type(list_files) is not tuple:
            raise WrongFormat()
        files = Path(self.files)
        files_in_trash = list(map(lambda x: x[0], self.list_files()))
        error = []
        for f in list_files:
            old_path = Path(f)
            name = old_path.name
            while name in files_in_trash:
                i = 1
                name += str(i)
                i += 1
            if not old_path.exists():
                error.append("file %s doesn't exist" % old_path.name)
                continue
            info_path = (Path(self.info) / (name + '.trashinfo'))
            info_path.touch()
            move(str(old_path.absolute()), str(files) + '/' + name)
            with open(str(info_path.absolute()), "w") as o:
                o.write('[Trash Info]\n')
                o.write('Path=%s\n' % str(old_path.absolute()))
                date = datetime.now().strftime("%Y-%m-%dT%H:%M:%S")
                o.write('DeletionDate=%s\n' % date)
        return error

    def restore(self, list_files):
        if type(list_files) is not list and type(list_files) is not tuple:
            raise WrongFormat()
        error = []
        files_dir = Path(self.files)
        info_dir = Path(self.info)
        for f in list_files:
            file_path = files_dir / f
            file_info = info_dir / (f + ".trashinfo")
            if not file_path.exists():
                error.append("file doesn't in %s trash" % self.name)
                continue
            if not file_info.exists():
                error.append(
                    "file doesn't have trahsinfo in %s trash" % self.name)
                continue
            with file_info.open() as o:
                i = list(o)
                old_path = i[1].split("Path=")[1][:-1]
                # move file to old path
                try:
                    move(str(file_path.absolute()), old_path)
                    file_info.unlink()
                except Exception as e:
                    error.append(str(e))
        return error

    def __iter__(self):
        """Trash object iter on file in /files"""
        return self.list_files()

    def __str__(self):
        """return path """
        return "%s" % (self.path)



import pytest
from trashtalk import Trash


@pytest.fixture()
def list_files():
    return ['file1.txt',
            'file2.txt',
            'file3.txt',
            'file4.txt',
            'file5.txt'
            ]


@pytest.fixture()
def list_files_size():
    return [['file1.txt', 4],
            ['file2.txt', 4],
            ['file3.txt', 4],
            ['file4.txt', 4],
            ['file5.txt', 4],
            ['Total: ', 20]
            ]


@pytest.fixture()
def generate_trash(tmpdir):
    d = tmpdir.mkdir('trash_test')
    m = tmpdir.mkdir('media')
    m.mkdir('.Trash-0000')
    files = tmpdir.mkdir('trash_test/files')
    info = tmpdir.mkdir('trash_test/info')
    tmp_desk = tmpdir.mkdir('desk')
    trash = Trash(str(d), 'trash_test')
    return trash



@pytest.fixture()
def trash_with_files(tmpdir, list_files, generate_trash):
    trash = generate_trash
    files = tmpdir.join(trash.name + '/files')
    info = tmpdir.join(trash.name + '/info')
    tmp_desk = tmpdir.join('desk')
    for f in list_files:
        files.join(f).write('osef')
        infostring = ('[Trash Info]\nPath=%s/%s\n'
                      'DeletionDate=Unknow\n') % (str(tmp_desk), f)
        info.join(f + '.trashinfo').write(infostring)
    return trash


@pytest.fixture()
def trash_with_dir_and_files(tmpdir, trash_with_files, list_files):
    trash = trash_with_files
    dir_test = tmpdir.mkdir(trash.name + '/files/testdir')
    info = tmpdir.join(trash.name + '/info')
    tmp_desk = tmpdir.join('desk')
    infostring = '[Trash Info]\nPath=%s/%s\nDeletionDate=Unknow\n' % (str(tmp_desk), 'testdir')
    info.join('testdir.trashinfo').write(infostring)
    for f in list_files:
        dir_test.join(f).write('osef')
    return trash



from __future__ import absolute_import
from trashtalk.trash import Trash
from trashtalk.core import trashtalk
from trashtalk.autocomplete_bash import autocomplete

__version__ = '0.0.6.1'

__all__ = ['__version__', 'Trash', 'trashtalk']



from __future__ import print_function, absolute_import
from os import getlogin
from trashtalk.core import parse_option
from trashtalk.generate_trashs import generate_trashs, get_media_trashs


def autocomplete(args=''):
    args = args.split()
    args = args[1:]
    if args:
        options, unknown = parse_option(args)
    else:
        options = parse_option(args)
    args.reverse()
    if (options.am or options.trash) and "home" not in options.trash:
        home = False
    else:
        home = True
    if options.a:
        options.am = True
    trashs, error = generate_trashs(options.u, options.trash,
                                  home, options.am)

    for arg in args:
        if arg[0] in '-':
            if arg == "-f":
                for trash in trashs:
                    for f in trash.list_files():
                        print(r"%s" % f[0])
            return
    for trash in get_media_trashs(getlogin())[0]:
        print(trash.name)



from tests.init_test import (generate_trash, list_files,
                             list_files_size, trash_with_dir_and_files,
                             trash_with_files)
from pathlib import Path
from datetime import datetime
from trashtalk.exception import WrongFormat
from trashtalk.trash import Trash
import pytest


def test_path_name(generate_trash):
    trash = generate_trash
    trash.select_path("/test", "test")
    assert str(trash) == "/test"
    assert trash.name == "test"
    assert trash.info == "/test/info"
    assert trash.files == "/test/files"
    t = Trash()
    assert t.name == ""


def test_iter(trash_with_files, list_files):
    """
    test list_files from trash
    """
    trash = trash_with_files
    # test all files in list_files
    for f in trash:
        assert f[0] in list_files


def test_list(trash_with_files, list_files):
    """
    test list_files from trash
    """
    trash = trash_with_files
    # test all files in list_files
    for f in trash.list_files():
        assert f[0] in list_files

    # test one select file
    f = trash.list_files([list_files[2]])
    assert list(f)[0][0] == list_files[2]

    # test wrong file
    f = trash.list_files(['error.txt'])
    assert list(f)[0][0] == None


def test_size(trash_with_files, list_files_size):
    """
    test list_files from trash with correct size in byte
    """
    trash = trash_with_files
    for f in trash.list_files(size=True):
        assert f in list_files_size


def test_info_list(trash_with_files, list_files, tmpdir):
    trash = trash_with_files
    desk = tmpdir.join('desk')
    for f in trash.list_files(info=True):
        assert f[0] in list_files
        assert f[1] == "%s/%s" % (str(desk), f[0])
        assert f[2] == "Unknow"


def test_clean(list_files, trash_with_dir_and_files):
    """
    test all files are removed: /files and /info
    """
    trash = trash_with_dir_and_files
    list_files = list_files
    files = Path(trash.path) / "files"
    info = Path(trash.path) / "info"

    # test for one delete file
    one_file = list_files.pop()
    error = trash.clean([one_file])
    one_file_path = files / one_file
    assert one_file_path.exists() == False
    one_file_info = info / (one_file + ".trashinfo")
    assert one_file_info.exists() == False

    #test error
    error = trash.clean(['error_file'])
    assert type(error[0]) == str
    assert len(error) == 1

    # test all cleaning
    trash.clean()
    for f in list_files:
        file_path = files / f
        assert file_path.exists() == False
        file_info = info / (f + ".trashinfo")
        assert file_info.exists() == False

    # test dir cleaning
    dir_path = files / 'testdir'
    assert dir_path.exists() == False
    dir_info_path = info / 'testdir.trashinfo'
    assert dir_info_path.exists() == False



def test_remove(list_files, generate_trash):
    """
    test file are remove with the correct info
    """
    trash = generate_trash
    desk = Path(trash.path + '/../desk')

    # create files and dir
    test_dir = desk / "dir"
    test_dir.mkdir()
    new_path = [str(test_dir.absolute())]
    for f in list_files:
        new_file = desk / f
        new_file.touch()
        new_path.append(str(new_file.absolute()))
        new_file = test_dir / f
        new_file.touch()
    # use remove method
    trash.remove(new_path)

    test_allready_in_trash = desk / list_files[0]
    test_allready_in_trash.touch()
    trash.remove([str(test_allready_in_trash.absolute())])
    # test
    now = datetime.now()
    trash_path = Path(trash.path)
    assert (trash_path / "files" / (
        test_allready_in_trash.name + '1')).exists() is True
    assert (trash_path / "info" / (
        test_allready_in_trash.name + '1.trashinfo')).exists() is True
    for path in new_path:
        p = Path(path)
        assert p.exists() is False
        assert (trash_path / "files" / p.name).exists() is True
        if p.name is not "dir":
            assert (Path(trash.path) / "files/dir" / p.name).exists() is True
        info = trash_path / "info" / (p.name + ".trashinfo")
        assert info.exists() is True
        with info.open(encoding="utf-8") as f:
            info_str = [i for i in f]
            assert info_str[0] == "[Trash Info]\n"
            assert info_str[1].split("=")[1][:-1] == path
            date = datetime.strptime(
                info_str[2].split('=')[1][:-1], "%Y-%m-%dT%H:%M:%S")
            assert now.year == date.year
            assert now.month == date.month
            assert now.day == date.day

    # test false info
    error = trash.remove(["/tmp/fakepath/nohing/arg.fail"])
    assert bool(error) is True
    assert (trash_path / "files" / "arg.fail").exists() is False
    assert (trash_path / "info" / "arg.fail.trashinfo").exists() is False
    with pytest.raises(WrongFormat):
        trash.remove("string is not accepted")


def test_restore(trash_with_files, list_files):
    trash = trash_with_files
    desk = (Path(str(trash)) / '..' / 'desk')
    path_files = (Path(str(trash)) / "files")
    path_info = (Path(str(trash)) / "info")
    trash.restore(list_files)
    for f in list_files:
        assert (desk / f).exists() is True
        assert (path_files / f).exists() is False
        assert (path_info / (f + ".trashinfo")).exists() is False
    with pytest.raises(WrongFormat):
        trash.restore("string is not accepted")
    error = trash.restore(list_files)
    assert bool(error) is True

    test_without_info = path_files / "file.test"
    test_without_info.touch()
    error = trash.restore([str(test_without_info.absolute())])
    assert bool(error) is True



from __future__ import print_function
import sys


def human_readable_from_bytes(num):
    if not num or type(num) is str:
        return num
    for unit in ['', 'K', 'M', 'G', 'T', 'P', 'E', 'Z']:
        if abs(num) < 1024:
            return ("%.2f" % num).rstrip('0').rstrip('.') + unit
        num /= 1024.0

    return ("%.2f" % num).rstrip('0').rstrip('.') + 'Y'


def print_files(list_files, nb_col=1):
    line = ""
    if not list_files:
        return
    if len(list_files[0]) > 1:
        for row in list_files:
            row[1] = human_readable_from_bytes(row[1])
    for e in range(nb_col):
        new_list = []
        for x in list_files:
            try:
                new_list.append(x[e])
            except:
                x.append("")
                new_list.append("")
        spaces = len(str(max(new_list, key=lambda x: len(str(x))))) + 1
        line += "{%d:%d}" % (e, spaces)
    for f in list_files:
        if f[0]:
            print(line.format(*f))
        else:
            print(f[1], file=sys.stderr)



from __future__ import print_function, absolute_import
import pwd
from os import getlogin
from pathlib import Path
from trashtalk.trash import Trash
import sys
from os.path import expanduser

"""
Module who generate trash

:example
in /home/user/.trashtalk
add
# to make a commentary
MEDIA_PATH=path/to/your/media
TRASH_PATH=a/direct/path/to/trash , nameofyourtrash
"""

MEDIA_DIR = ['/media']
TRASHS_PATH = [] # direct link to Trash
HOME_TRASH_PATH = ".local/share/Trash"


def add_profil_info(f):
    for line in f:
        line = line.split('#')[0].strip()
        try:
            key, val = line.split("=")
            if key == "MEDIA_PATH":
                MEDIA_DIR.append(val)
            elif key == "TRASH_PATH":
                path, name = val.split(',')
                TRASHS_PATH.append((name.strip(), path.strip()))
        except:
            pass

profil = Path(expanduser("~/.trashtalk"))
if profil.exists():
    add_profil_info(profil.open())

def generate_trashs(users=[], medias=[], home=True, all_media=False):
    trashs = []
    error = []
    if not users:
        users = [getlogin()]
    for user in users:
        if home:
            trash = Path(expanduser("~%s/%s" % (user, HOME_TRASH_PATH)))
            if trash.exists():
                trashs.append(Trash(str(trash.absolute()), user))
            else:
                error.append("can't find: " + trash.name)
        if all_media or medias:
            trashs_media, errors_media = get_media_trashs(user, medias)
            trashs += trashs_media
            error += errors_media
    return trashs, error


def get_media_trashs(user, medias_name=[]):
    trashs = []
    error = []
    media_dir = MEDIA_DIR + ["%s/%s" % (MEDIA_DIR[0], user)]
    for d in media_dir:
        m_d = Path(d)
        if not m_d.exists():
            continue
        for m in m_d.iterdir():
            trash = m /  (".Trash-" + str(pwd.getpwnam(user)[2]))
            if trash.exists() and (not medias_name or m.name in medias_name):
                trashs.append(Trash(str(trash.absolute()), m.name))
            elif m.name in medias_name:
                error.append("can't find: " + m.name)
    for d in TRASHS_PATH:
        trash = Path(d[1])
        name = d[0]
        if trash.exists() and (not medias_name or name in medias_name):
            trashs.append(Trash(str(trash.absolute()), name))
        elif name in medias_name:
            error.append("can't find: " + name)
    return trashs, error



from pathlib import Path
from trashtalk import generate_trashs
from tests.init_test import generate_trash
import pwd


def test_add_profil_info(tmpdir):
    test_file = tmpdir.join('.trashtalk')
    s = "MEDIA_PATH=/testmediapath\nTRASH_PATH=/testtrashpath , bob"
    test_file.write(s)
    f = Path(str(test_file))
    generate_trashs.MEDIA_DIR = ['/media']
    generate_trashs.TRASHS_PATH = []
    generate_trashs.add_profil_info(f.open())
    assert generate_trashs.MEDIA_DIR == ['/media', '/testmediapath']
    assert generate_trashs.TRASHS_PATH == [('bob', '/testtrashpath')]


def test_get_media_trashs(generate_trash, tmpdir, monkeypatch):
    trash = generate_trash
    generate_trashs.MEDIA_DIR = [str(tmpdir)]
    generate_trashs.TRASHS_PATH = [("test", trash.path)]
    def mockgetpwnam(user):
        return [1, 2, '0000']
    monkeypatch.setattr(pwd, 'getpwnam', mockgetpwnam)
    trashs, err = generate_trashs.get_media_trashs("remy")
    assert trashs[0].path == str(tmpdir) + "/media/.Trash-0000"
    assert trashs[0].name == "media"
    assert trashs[1].path == trash.path
    assert trashs[1].name == "test"
    trashs, err = generate_trashs.get_media_trashs("remy", ['desk'])
    assert trashs == []
    assert bool(err) is True
    generate_trashs.TRASHS_PATH = [("error", str(tmpdir) + "/fail")]
    trashs, err = generate_trashs.get_media_trashs("remy", ['error'])
    assert trashs == []
    assert bool(err) is True



#!  /usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
from sys import version_info


requires = ['pathlib==1.0.1']

# bash completion
try:
    with open('/etc/bash_completion.d/completion', 'w') as eo:
        eo.write('test')
    data_files = [('/etc/bash_completion.d/', ['extra/python%s/trashtalk' % version_info[0]])]
except:
    data_files = []

setup(
    name='trashtalk',
    version="0.0.6.1",

    packages=find_packages(),

    setup_requires=['pytest-runner'],
    tests_require=['pytest'],

    description="simplify trash in command line",
    long_description=open('README.rst').read(),
    author="PTank",
    author_email="akazian@student.42.fr",
    url="https://github.com/PTank/trashtalk",

    install_requires=requires,

    include_package_data=True,

    classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: POSIX",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 2",
        "Topic :: Utilities"
    ],

    data_files=data_files,

    entry_points={
        'console_scripts': [
            'trashtalk = trashtalk.core:trashtalk',
        ],
    }

)
